#include <stdio.h>

void intro();
void init_pos();
 
void intro()
{
    printf("W E L C O M E   T O   T H E   W O R L D   O F");
    printf("   P L A T E A U (TM)\n\n");
    printf("PLATEAU, Copyright Magnolia Game Co., Inc. 1986, 1990, 1991,");
    printf(" 1992.\n\n");
    printf("PLATEAU is a copyrighted board game produced by");
    printf(" Magnolia Game Co. Inc.\n");
    printf("The computer implementation of the game is licenced");
    printf(" under special\n");
    printf("permission to James Kosinski by Magnolia Games.  ");
    printf("NO OTHER VERSIONS OF\n");
    printf("PLATEAU ARE PERMITTED WITHOUT WRITTEN PERMISSION BY");
    printf(" MAGNOLIA GAMES.\n\n");
    printf("PLATEAU board game sets may be ordered from Magnolia Games,");
    printf(" by sending\n");
    printf("$16.00 to P.O. Box 143, Madison, AL  35758   USA.\n\n");
    printf("This computer implementation, HONOR PLATEAU, Copyright");
    printf(" James Kosinski, 1992.\n");
    printf("It may be copied and distributed provided this header");
    printf(" remains intact and\n");
    printf("the code is not modified except for minor changes to get it ");
    printf("\"up and running.\"\n\n");
    printf("If you use this program, you are encouraged to send");
    printf(" $10.00 to:\n");
    printf("James Kosinski, 4907 Highland Dr., Berrien Springs  ");
    printf("MI  49103-1051.\n\n");
    printf("Join the Plateau Newsletter mailing list!  Just send a");
    printf(" message to:\n");
    printf("plateau-request@plateau.com  or  ");
    printf("...uunet!plateau!plateau-request\n\n");
    printf("PLATEAU is a trademark of Magnolia Game Co., Inc.\n");
}

void init_pos(fptr)
FILE *fptr;
{   
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 01 15 00 99 00 03\n");
    fprintf(fptr,"00 02 10 00 99 00 03\n");
    fprintf(fptr,"00 03 08 00 99 00 03\n");
    fprintf(fptr,"02 02 05 00 99 00 03\n");
    fprintf(fptr,"02 02 05 00 99 00 03\n");
    fprintf(fptr,"02 03 21 00 99 00 03\n");
    fprintf(fptr,"03 03 04 00 99 00 03\n");
    fprintf(fptr,"03 03 04 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 00 01 00 99 00 03\n");
    fprintf(fptr,"00 01 15 00 99 00 03\n");
    fprintf(fptr,"00 02 10 00 99 00 03\n");
    fprintf(fptr,"00 03 08 00 99 00 03\n");
    fprintf(fptr,"02 02 05 00 99 00 03\n");
    fprintf(fptr,"02 02 05 00 99 00 03\n");
    fprintf(fptr,"02 03 21 00 99 00 03\n");
    fprintf(fptr,"03 03 04 00 99 00 03\n");
    fprintf(fptr,"03 03 04 00 99 00 03\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
    fprintf(fptr,"00 99\n");
}
